from processus import Processus

class Ordonnanceur:
    """
    Classe générale modélisant un ordonnanceur.
    Les classes qui héritent de celle-ci doivent implémenter une méthode 'inserer' qui insère un processus dans le file d'exécution selon la méthode suivie par l'algorithme d'ordonnancement.
    """

    def __init__(self, processus: list[Processus]):
        """
        processus : liste de processus à ordonnancer (list[Processus])
        file_execution : file d'exécution de l'ordonnanceur (list[Processus])
        ticks : tableau des processus éxécutés à chaque "tick" d'horloge (list[Processus])
        temps : décompte des "ticks" d'horloge écoulés depuis le début de l'ordonnancement (int)
        """
        self.processus = processus
        self.file_execution = []
        self.ticks = []
        self.temps = 0

    def __str__(self)->str:
        """
        Méthode de représentation textuelle de la suite d'éxécution des processus à l'issue de l'ordonnancement.
        Selon l'environnement (IDE ou en ligne), l'affichage diffère un peu.
        """
        import sys

        if sys.platform == 'emscripten': # en ligne ?
            t = ""
            for i in range(len(self.ticks)):
                t += f"{i} : {self.ticks[i].id}\n"
            return t
        else: # IDE ?
            t = ""
            p = ""
            for i in range(len(self.ticks)):
                t += f"{i}\t"
                p += f"{self.ticks[i].id}\t"
            return t + "\n" + p

    def inserer(self, p: Processus):
        """
        Insère un processus dans la file d'execution.
        A IMPLEMENTER SELON UN ALGORITHME D'ORDONNANCEMENT DONNE.
        """
        pass

    def tick(self):
        """
        Simule l'éxécution d'un "tick" d'horloge du CPU
        """
        # un nouveau processus ( s'il en reste ! ) arrive ?
        if len(self.processus) != 0:
            p = self.processus[0]
            if p.date_arrivee == self.temps:
                self.inserer(p)
                self.processus.pop(0)

        # éxécution du processus élu
        elu = self.file_execution[0]
        elu.executer()

        # tick: elu
        self.ticks.append(elu)

        # elu terminé ?
        if elu.est_termine():
            self.file_execution.pop(0)

        self.temps += 1


    def ordonnancement(self):
        """
        Simule l'éxécution d'un algorithme d'ordonnancement.
        """
        self.tick()
        while len(self.file_execution) != 0:
            self.tick()

